/**
* \file logger.h
*/

#ifndef LOGGER_H_
#define LOGGER_H_

#include <fstream>
#include <string>
#include <sstream>
#include <map>

#define ENUM_TO_STR(enumValue) #enumValue

using namespace std;

namespace logger {

/**
* \brief Nvel de mensagem.
* O nvel ALL e OFF possuem um comportamento distinto  perante os demais. ALL representa  
* todos  os demais nveis com exceo do OFF. O  nvel   OFF  representa  a  ausncia de 
* todos os demais nveis com exceo do ALL.
*/
  enum Level {
    ALL,
    ERROR,
    INFO,
    WARNING,
    OFF
  };

/**
* \brief Representa um objeto do tipo Logger  a   ser utilizado como mecanismo de log.
*/ 
  class Logger {
    private:
    /**
    * Relao entre os nveis como elementos de enumerao com as representaes dos nveis em 
    * strings.
    */
      static const char* levelStr[];

    /**
    * Singleton do Logger.
    */
      static Logger* logger;

    /**
    * Mapeamento que descreve quais nveis esto ativados.
    */
      map<Level, bool> levelFlag; 

    /**
    * Contador de edentao utilizado para se gerar   uma sada hierrquica conforme a pilha de 
    * chamadas.
    */
      short numIndent;

    /**
    * Path relativo ou absoluto do arquivo  de  sada  do Logger.
    */
      char* filename;
      ostream* output;

      Logger();
      ~Logger();
    public:
      
    /**
    * Fornece a nica instncia do Logger.
    *
    * @return Logger
    */
      static Logger* getInstance();
   
    /**
    * Define a sada do logger.
    *
    * O padro  sada padro stdout.
    *
    * @param[in] filename Caminho relativo ou absoluto do arquivo de sada do logger. 
    * O valor 0 (NULL) representa a sada padro stdout.
    */
      void setOutput(char* filename);

    /**
    * Ativa ou desativa um determinado nvel de mensagem.
    *
    * Se o nvel est desativado, a chamada a este mtodo ativar o mesmo, e vice-versa.
    *
    * @param [in] Nvel de mensagem.
    */
      void setLevel(Level level);

    /**
    * Informa se um nvel est ativo ou no.
    *
    * @return True se o nvel est ativo, caso contrrio retorna false.
    */
      bool getLevel(Level level);

    /**
    * Registra uma mensagem utilizando um determinado nvel.
    *
    * @param [in] level Nvel de mensagem.
    * @param [in] message Mensagem.
    */
      void log(
        Level level, 
        string message);

    /**
    * Edenta em um nvel as mensagens seguintes.
    *
    * Abre-se um novo escopo para as mensagens seguintes a esta chamada.
    */
      void indent(); 

    /**
    * Edenta em um nvel as mensagens seguintes.
    *
    * Registra uma mensagem e abre um novo escopo para as mensagens seguintes a esta chamada.
    *
    * @param [in] level Nvel de mensagem.
    * @param [in] message Mensagem.
    */
      void indent(
        Level level, 
        string message);

    /**
    * Desfaz uma edentao.
    *
    * Fecha o escopo atual de mensagens.
    */
      void dedent();

    /**
    * Desfaz uma edentao.
    *
    * Fecha o escopo atual de mensagens e logo aps registra uma mensagem.
    *
    * @param [in] level Nvel de mensagem.
    * @param [in] message Mensagem.
    */
      void dedent(
        Level level, 
        string message);
  };
}

#endif
